  
  ; /---------------------------------------------------------------------\ ;
  ; +---------------------------------------------------------------------+ ;
  ; |  SSFS 00.01 (Schematically Structured File System) BootStrap 0.1.2  | ;
  ; |                                                                     | ;
  ; |   Coded by: Vandad Nahavandipoor                                    | ;
  ; |   Email   : AlexiLaiho.cob@Gmail.com                                | ;
  ; |   Web-site: Http://www.AsmTrauma.com                                | ;
  ; |   Coded on: July 12th 2007 at 5:17 AM                               | ;
  ; |   Coded in: NASM version 0.98.39 compiled on Jan 16 2005            | ;
  ; |                                                                     | ;
  ; |                                                                     | ;
  ; |   Changes made to version 0.1.2 (Previous version = 0.1.1):         | ;
  ; |      Aligned the Stack Pointer on a DWORD boundary.                | ;
  ; |       Thanks to [bughunter] for pointing this out.                  | ;
  ; |      Explicitly set SS:SP to 0x0000:0xFFFC.                        | ;
  ; |      Explicitly set ES:DI for the MOVSW insrtuction to             | ;
  ; |       0x0000:0x7C00. The ES segment register was not set to         | ;
  ; |       0x0000 explicitly in the previous versions.                   | ;
  ; |       Thanks to [urxae] for pointing this out.                      | ;
  ; |      Fixed some typos in the comment lines.                        | ;
  ; |                                                                     | ;
  ; |                                                                     | ;
  ; |   Changes made to version 0.1.1 (Previous version = 0.1):           | ;
  ; |      Removed an ineffective JMP instruction (Saved 2 bytes).       | ;
  ; |      This bootstrap now passes the pointer to the current          | ;
  ; |       active partition in the SI register to the boot sector        | ;
  ; |       that will be loaded and jumped to, at 0x0000:0x7C00.          | ;
  ; |       Thanks to frank (http://fuzzylogicos.googlepages.com/)        | ;
  ; |       for pointing this problem out.                                | ;
  ; |                                                                     | ;
  ; |                                                                     | ;
  ; |   SSFS 00.01 BootStrap 0.1's tasks:                                 | ;
  ; |      Relocates the HDD's MBR into memory at 0x0000:0x0600.         | ;
  ; |      Searches in the Partition Table of the HDD to find            | ;
  ; |       an active partition (Flag = 0x80).                            | ;
  ; |      If an active partition is not found, it will display a        | ;
  ; |       relevant error message.                                       | ;
  ; |      If an active partition is found, it will check its            | ;
  ; |       Starting C/H/S value to see if it is valid.                   | ;
  ; |      If the Starting C/H/S value is valid, it will load the        | ;
  ; |       Boot Sector of that drive into memory at 0x0000:0x7C00 using  | ;
  ; |       INT 0x13 (Disk Operations).                                   | ;
  ; |      If the Starting C/H/S value is not valid, it will             | ;
  ; |       use IDE ports to load the Boot Sector of that partition       | ;
  ; |       using its LBA28/32 address to the memory location at          | ;
  ; |       0x0000:0x7C000.                                               | ;
  ; |      After the Boot Sector of the active partition is loaded       | ;
  ; |       in 0x0000:0x7C00, it will check for the Boot Sector's         | ;
  ; |       Boot Signature (0xAA55).                                      | ;
  ; |      If the Boot Signature of the active partition's Boot Sector   | ;
  ; |       is valid, it will jump to the Boot Sector and give the        | ;
  ; |       control to it.                                                | ;
  ; |      If the Boot Signature of the active partition's Boot Sector   | ;
  ; |       is not valid, it will display an error message.               | ;
  ; |                                                                     | ;
  ; |                                                                     | ;
  ; |   SSFS 00.01 BootStrap 0.1.2's code details:                        | ;
  ; |      Compiled code's size in bytes = 446.                          | ;
  ; |      Used bytes                    = 345.                          | ;
  ; |      Free bytes filled with null   = 101 (Free to program)         | ;
  ; |      Instruction set used          = x86.                          | ;
  ; |      Original code location        = CS:0x7C00.                    | ;
  ; |      Relocated code location       = 0x0000:0x0600.                | ;
  ; |      Loads the boot sector at      = 0x0000:0x7C00.                | ;
  ; |      Uses stack at                 = 0x0000:0xFFF.                 | ;
  ; |                                                                     | ;
  ; |                                                                     | ;
  ; |   Disclaimer:                                                       | ;
  ; |     I take absolutely no responsibility for what you might do with  | ;
  ; |     this bootstrap code. You might damage your Hard Disk Drive's    | ;
  ; |     partition table by inaccurately using this program. I recommend | ;
  ; |     that you understand what Master Boot Records are all about      | ;
  ; |     before using this bootstrap code for your HDD's MBR.            | ;
  ; +---------------------------------------------------------------------+ ;
  ; \---------------------------------------------------------------------/ ;
  
  
[BITS 16]                                                               ; Use 16 Bit operands and instructions
[ORG  0x0600]                                                           ; 0x0000:0x7C00 is where the BIOS loads the MBR into the Memory
                                                                        ; But since we are going to relocate our program to 0x0000:0x0600, then
                                                                        ; our best bet is to set the origin point of the program to 0x0600.
                                                                        ; Care must be taken not to reference invalid memory points before the relocation
                                                                        ; since the memory location in that point is set to 0x0000:0x0600 instead of the
                                                                        ; actual origin which is 0x7C00.
; 
%MACRO INVOKE 1-*                                                       ; The INVOKE macro that calls procedures using the STDCALL calling convention
  %REP %0 - 1                                                           ; Repeat as many times as the number of parameters
    %ROTATE -1                                                          ; Move the rightmost parameter to the left
    PUSH    WORD %1                                                     ; Push that parameter onto the stack
  %ENDREP                                                               ; End the repeating
  %ROTATE -1                                                            ; Now get the name of the procedure
  CALL    %1                                                            ; Call that name
%ENDMACRO                                                               ; End of the INVOKE macro
; 
%DEFINE   PTR                                                           ; A dummy definition for the PTR keyword like in MASM
%DEFINE   OFFSET                                                        ; Another dummy definition for the OFFSET keyword like in MASM
%INCLUDE "IDEConstants.inc"                                             ; Include all the IO addresses of IDE Controllers and bit values
; 
[SECTION .text]                                                         ; Code section
START:                                                                  ; Start the code section






                                                                        ; <CODE BLOCK>
                                                                        ; Purpose = Set the segment registers as soon as the program is executed
  MOV     AX , CS                                                       ; Read CS in AX
  MOV     DS , AX                                                       ; DS = CS
  XOR     AX , AX                                                       ; Set the accumulator to 0x0000 to be moved to SS and ES
  MOV     SS , AX                                                       ; SS = 0x0000
  MOV     ES , AX                                                       ; ES = 0x0000 to later be used with the MOVSW instruction (Destination = ES:0x0600)
  MOV     SP , 0xFFFC                                                   ; SS:SP = 0x0000:0xFFFC at the physical address of 0x0000FFFC
                                                                        ; </CODE BLOCK>





  
                                                                        ; <CODE BLOCK>
                                                                        ; Purpose = Relocate the MBR that is loaded by BIOS in 0x0000:0x7C00 to
                                                                        ;           0x0000:0x0600
  
  MOV     SI , 0x7C00                                                   ; The source address is 0x7C00, where BIOS loads our MBR into the memory
  MOV     DI , 0x0600                                                   ; The destination is 0x0600, where we want to relocate our entire MBR (512 bytes)
  CLD                                                                   ; Clear the direction flag so that we will go forward while copying
  MOV     CX , 512 / 2                                                  ; We are going to copy 2-bytes at each iteration thus 512 / 2 bytes
  REP     MOVSW                                                         ; Copy from DS/CS:0x7C00 to ES(0x0000):0x0600, 2 bytes at a time
  JMP     0x0000:0x0600 + (.RestofTheCode - $$)                         ; Now calculate the address where we have to jump to (RestofTheCode)
                                                                        ; </CODE BLOCK>



  


                                                                        ; <CODE BLOCK>
                                                                        ; Purpose = Set the partition count and paritions' descriptor address in memory
  .RestofTheCode:                                                       ; The rest of the code will resume at 0x0000:0x0600 + the offset of this label
    MOV     CX , 0x0004                                                 ; There are the maximum number of 4 partitions in the MBR
    MOV     BX , 0x0600 + 446                                           ; DS:BX = Our current memory location + the offset of the first partition's
                                                                        ; descriptor in the MBR. We will keep this value in BX whenever we need to read
                                                                        ; a value from the MBR that is related to a partition.
                                                                        ; 0x0600 is where we relocate our MBR after it is loaded into 0x7C00 by BIOS and
                                                                        ; +446 is the constant offset of the descriptor of the first partition in the MBR
                                                                        ; </CODE BLOCK>






                                                                        ; <CODE BLOCK>
                                                                        ; Purpose = Find an Active partition
  .FindActivePartition:                                                 ; Try to search in all 4 partitions to find an active one
    MOV     AL , BYTE PTR [DS:BX]                                       ; Read the boot indicator of the partition
    CMP     AL , 0x80                                                   ; See if the partition is active
    JE      .FoundAnActivePartition                                     ; Jump to ... if the current partition is active
    ADD     BX , 0x0010                                                 ; Move to the next partition descriptor in the MBR
    DEC     CX                                                          ; Decrement the number of partition descriptors in the MBR
    JNZ     .FindActivePartition                                        ; Keep trying if we still have partitions to search
  .CouldNotFindActivePartition:                                         ; Here we are sure that there were no active partitions in the MBR
    INVOKE  __Panic , OFFSET ERRNoBootablePartition                     ; Write a message to the console and halt the system if an active
                                                                        ; </CODE BLOCK>






                                                                        ; <CODE BLOCK>
                                                                        ; Purpose = See if the starting CHS value of the current partition is valid
                                                                        ; Invalid CHS:
                                                                        ;    Cylinder = 1023
                                                                        ;    Head     = 254
                                                                        ;    Sector   = 63
                                                                        ; CHS is ordered in HSC in the MBR so the 3 invalid bytes will be 0xFE, 0xFF, 0xFF
                                                                        ; DS:BX = Location of the current partition's descriptor in the memory
  .FoundAnActivePartition:                                              ; We have now found an active partition
    MOV     AX , WORD PTR [DS:BX + 1]                                   ; Read the value of the Head and the Sector of the partition
    MOV     SI , AX                                                     ; Make a copy of AX for future uses
    XOR     CX , CX                                                     ; Clear the CX register
    MOV     CL , BYTE PTR [DS:BX + 3]                                   ; CL = The Cylinder value of the starting CHS of the current partition
    MOV     DI , CX                                                     ; Make a copy of CX for future uses
    XOR     AX , 0xFFFE                                                 ; XOR AX with its maximum value for Head and Sector
    XOR     CX , 0x00FF                                                 ; XOR CX with its maximum value for Cylinder
    OR      AX , CX                                                     ; See if both of them are zero because if they are then CHS is invalid
    JNZ     .UseCHSToReadTheBootSectorOfTheActivePartition              ; Jump to ... if not (CHS is valid)
    JMP     .UseLBA28ToReadTheBootSectorOfTheActivePartition            ; Jump to ... if yes (CHS is invalid, Use LBA28 instead)
                                                                        ; </CODE BLOCK>






                                                                        ; <CODE BLOCK>
                                                                        ; Purpose = Read the Boot Sector of the current partition in 0x0000:0x7C00
                                                                        ;           And jump to that location. Reading will be done using C/H/S
                                                                        ;           and INT 0x13
    .UseCHSToReadTheBootSectorOfTheActivePartition:                     ; Use C/H/S translation to read from the HDD
      PUSH    BX                                                        ; BX is the pointer to the current partition in memory. We are going to have
                                                                        ; to keep this pointer to later pass it in the SI register to the boot sector
                                                                        ; of the partition that we will load at 0x0000:0x7C00.
      STI                                                               ; Enable the maskable interrupts
      MOV     BP , 0x000B                                               ; all GPRs are in use, use BP as the timeout-counter (10 READ ATTEMPTS)
      XOR     BX , BX                                                   ; We should read the Boot Sector to 0x0000:0x7C00, so set the segment selector
      MOV     ES , BX                                                   ; Set the segment selector ES to 0x0000. Reading will be done on ES:BX
      MOV     BX , 0x7C00                                               ; Set the offset into the segment 0x0000:0x7C00
      .ReadFromDiskAgainCHS:                                            ; Start setting proper parameters for INT 0x13
        DEC     BP                                                      ; Decrement the timeout-counter once
        JZ      .ErrorInReadingCHS                                      ; Jump to ... if time is up. We are not going to try anymore
      .ResetDiskSystem:                                                 ; Reset the disk systems before reading anything from the HDD
        XOR     AH , AH                                                 ; Function (RESET DISK SYSTEM); before reading, reset the disk
        MOV     DL , 0x80                                               ; Set Bit#7 to reset both the floppy and the hard disk
        INT     13                                                      ; Issue the interrupt for resetting the disk systems
        JC      .ReadFromDiskAgainCHS                                   ; Decrement the counter again if an error has occurred
        MOV     AX , SI                                                 ; SI was a copy of AX so return it back to AX
        MOV     CX , DI                                                 ; DI was a copy of CX in the above code so put it back in its place
        MOV     CH , CL                                                 ; CH = Cylinder Low is in CL so put it in CH
        MOV     CL , AH                                                 ; CL = Sector Number that is inside AH (Plus 2 leftmost bits for Cylinder High)
        MOV     DH , AL                                                 ; DH = Head Number that is inside AL
        MOV     DL , 0x80                                               ; DL = Drive to read from (Bit#7 = 1 = HDD, Bits0..6 = 0 = Drive 0)
        MOV     AH , 0x02                                               ; Function (READ SECTOR(S) INTO MEMORY)
        MOV     AL , 0x01                                               ; AL = Sector Count (Only read one sector (The Boot Sector))
        INT     0x13                                                    ; Issue the interrupt
        JC      .ReadFromDiskAgainCHS                                   ; Keep reading for as many as BP times if an error has occurred
      MOV     AX , WORD PTR [DS:0x7C00 + 510]                           ; Now read the Boot Signature of the boot sector that we read into memory
      CMP     AX , 0xAA55                                               ; The boot signature must be 0xAA55
      JNE     .InvalidBootSectorCHS                                     ; Jump to ... if the boot signature is not valid
      POP     SI                                                        ; We had saved the pointer to the current partition in BX; restore it in SI so that
                                                                        ; we will be able to pass it to the boot sector at 0x0000:0x7C00
      JMP     0x0000:0x7C00                                             ; Now that we are sure that we have read the boot sector, jump to its code
      .ErrorInReadingCHS:                                               ; Here we are sure that all our attempts to read the boot sector failed
        INVOKE  __Panic, OFFSET ERRDiskRead                             ; Tell the user that we couldn't read the disk
      .InvalidBootSectorCHS:                                            ; Here we are sure that the boot signature of the boot sector was invalid
        INVOKE  __Panic, OFFSET ERRInvalidBootSector                    ; Tell the user about the invalidity of the Boot Sector
                                                                        ; </CODE BLOCK>






                                                                        ; <CODE BLOCK>
                                                                        ; Purpose = Read the Boot Sector of the current partition into 0x0000:0x7C00
                                                                        ;           using IDE ports for Drive 0 on the Primary IDE Controller
                                                                        ; BX = holds the structure of the current partition (SHOULD NOT BE CORRUPTED)
    .UseLBA28ToReadTheBootSectorOfTheActivePartition:                   ; Start reading from HDD0 on Primary IDE Controller using LBA28 translation
      MOV     AL , 0x01                                                 ; Setting the sector count to 1
      MOV     DX , PIDEC_PORT_SECTORCOUNT                               ; Set the output register to the Sector Count Register
      OUT     DX , AL                                                   ; Tell the Primary IDE controller that want to read 1 sector

      MOV     AX , WORD PTR [DS:BX + 0x08]                              ; Offset +0x08 from the current partition is LBA bits 0 to 15
      MOV     DX , PIDEC_PORT_LBABITS0TO7                               ; Set the output port to LBA Bits 0 to 7
      OUT     DX , AL                                                   ; Just set LBA BITS 0 to 7 for now

      MOV     AL , AH                                                   ; Now get the LBA bits 8 to 15 in AL
      MOV     DX , PIDEC_PORT_LBABITS8TO15                              ; Set the output port to the corresponding port
      OUT     DX , AL                                                   ; Set LBA bits 8 to 15

      MOV     AX , WORD PTR [DS:BX + 0x0A]                              ; Read LBA bits 16 to 31 from the current partition's structure at offset +0x0A
      MOV     DX , PIDEC_PORT_LBABITS16TO23                             ; Set the output port to LBA bits 16 to 23
      OUT     DX , AL                                                   ; Set LBA bits 16 to 23

      MOV     AL , AH                                                   ; Now get LBA bits 24 to 31 in AL (We will only use 4 more bits for LBA28)
      OR      AL , IDE_DRIVEHEADREG_DEFAULTVALUE | IDE_DRIVEHEADREG_L   ; Use LBA translation and select the default value for the Device/Head register
      MOV     DX , PIDEC_PORT_DRIVEHEAD                                 ; Set the output port to Device/Head Register
      OUT     DX , AL                                                   ; Put LBA bits 24 to 27 and default values in Device/Head Register

      MOV     DX , PIDEC_PORT_COMMAND                                   ; Now we should send the Read Sector(s) With Retry command to the command register
      MOV     AL , MIDE_CMD_READSECTORSWR                               ; Select the command
      OUT     DX , AL                                                   ; Send it to the IDE Controller

      XOR     AX , AX                                                   ; When data is ready, BSY should be 0 and DRQ should be 1
      MOV     DX , PIDEC_PORT_STATUS                                    ; We are going to read from the Status Register
      MOV     CX , IDE_TYPICAL_TIMEOUT                                  ; The total number of times that we are going to try to read from IDE
      .CheckStatusForLBA28:                                             ; Try to check the Status Register
        IN      AL , DX                                                 ; Read 1 byte from the Primary IDE Controller's status register
        AND     AL , (IDE_STATUSREG_DRQ | IDE_STATUSREG_BSY)            ; Keep only the BSY and the DRQ bits
        XOR     AL , IDE_STATUSREG_DRQ                                  ; If we XOR the BSY bit with the result, it must yield zero. Else, retry
        JZ      .ReadASectorInLBA28                                     ; If the result is zero, then start reading the boot sector of the partition
        DEC     CX                                                      ; Decrement the timeout-counter
        JNZ     .CheckStatusForLBA28                                    ; Keep trying if the timeout is not expired yet
      INVOKE  __Panic, OFFSET ERRDiskRead                               ; Now we know the sector was not transferred, so output a message

      .ReadASectorInLBA28:                                              ; Here we are going to read 512 bytes from the IDE Controller
        CLI                                                             ; Disable Maskable Interrupts while we are reading from IDE
        MOV     DX , PIDEC_PORT_DATA                                    ; Set the port address from which we read (The Data Port)
        CLD                                                             ; Clear the direction flag so that we will move forward in our buffer
        MOV     CX , 256                                                ; We will read 2 bytes at a time therefore, 512 / 2 = 256 WORDs
        XOR     AX , AX                                                 ; Set AX to zero
        MOV     ES , AX                                                 ; ES = 0x0000
        MOV     DI , 0x7C00                                             ; ES:DI = Our destination buffer = 0x0000:0x7C00
        REP     INSW                                                    ; Read from the data port of the primary IDE Controller
        NOP                                                             ; Just a small pause before we read the status register again

      STI                                                               ; Enable Maskable Interrupts
      MOV     DX , PIDEC_PORT_STATUS                                    ; Reading the Status Register will negate INTRQ
      IN      AL , DX                                                   ; Read the Status Register of the primary IDE Controller
      MOV     AX , WORD PTR [DS:0x7C00 + 510]                           ; Read the boot signature of the boot sector that we loaded into memory
      CMP     AX , 0xAA55                                               ; The boot signature must be 0xAA55
      JNE     .InvalidBootSectorLBA28                                   ; Jump to ... if the boot signature is not valid
      MOV     SI , BX                                                   ; While we read from the HDD using IDE ports, the BX register will not be destroyed
                                                                        ; and this register (BX) points to the memory location of the current partition
                                                                        ; that should be passed to the boot sector
      JMP     0x0000:0x7C00                                             ; Jump to the memory location where we read the Boot Sector of the parition
      .InvalidBootSectorLBA28:                                          ; Here we are sure that the boot signature was not valid
        INVOKE  __Panic, OFFSET ERRInvalidBootSector                    ; Tell the user about the invalidity of the boot signature
                                                                        ; </CODE BLOCK>



  JMP     $



                                                                        ; <CODE BLOCK>
                                                                        ; Purpose = Write 16-bit WORD values to the screen.
                                                                        ; [__WriteInt] is only used for debugging and only if necessary.
                                                                        ; Removing it will save 45 bytes of free space for code in the MBR
   ;__WriteInt:
   ; ; void __WriteInt (WORD InInteger); StdCall;
   ; PUSH    AX                                                          ; Push the accumulator onto the stack
   ; PUSH    BX                                                          ; Preserve the base index register's vlaue
   ; PUSH    CX                                                          ; Push the count register onto the stack
   ; PUSH    DX                                                          ; Push the data register onto the stack
   ; PUSH    BP                                                          ; Push the base pointer onto the stack
   ; MOV     BP , SP                                                     ; Move the stack pointer to the base pointer and make the stack frame
   ; MOV     AX , WORD PTR [BP + 0x0C]                                   ; *AX = The [InInteger] parameter
   ; MOV     CX , 0x000A                                                 ; Dividing the number by 10 and taking the remainder is what we should do
   ; PUSH    OFFSET .EP                                                  ; Push the offset of the exit point onto the stack
   ; .ConvertToChar:                                                     ; Start extracting digits
   ;   XOR     DX , DX                                                   ; DX:AX = The number to be divided by 10
   ;   DIV     CX                                                        ; Divide DX:AX by 10
   ;   OR      DL , 0x30                                                 ; Convert the remainder to a character
   ;   PUSH    DX                                                        ; Push the character onto the stack
   ;   TEST    AX , AX                                                   ; See if the number is zero yet
   ;   JZ      .Extract                                                  ; Start extracting the remainders if yes
   ;   CALL    .ConvertToChar                                            ; Keep dividing DX:AX by 10
   ; .Extract:                                                           ; Start extracting the pushed remainders
   ;   POP     AX                                                        ; Pop the last pushed remainder
   ;   MOV     AH , 0x0E                                                 ; We are going to print
   ;   INT     0x10                                                      ; Print the remainder that is converted to a character
   ;   RET                                                               ; Keep extracting
   ; .EP:                                                                ; End of the procedure and the exit point
   ;   POP     BP                                                        ; Restore the base pointer
   ;   POP     DX                                                        ; Restore the data register
   ;   POP     CX                                                        ; Restore the count register
   ;   POP     BX                                                        ; Restore the base index
   ;   POP     AX                                                        ; Restore the accumulator
   ; RET     0x02                                                        ; Return to the calling procedure
   ;                                                                     ; And sweep 1 parameter off the stack
                                                                        ; </CODE BLOCK>






  __Panic:
    ; void __Panic (const char* String); StdCall;
    MOV     BP , SP                                                     ; Move the stack pointer to the base pointer and make the stack frame
    MOV     BX , WORD PTR [BP + 0x02]                                   ; *BX = The [String] parameter
    MOV     AH , 0x0E                                                   ; The Print Character function
    .WriteString:                                                       ; Start searching the string for a null-byte
      MOV     AL , BYTE PTR [BX]                                        ; Read one byte from the string
      TEST    AL , AL                                                   ; See if it is the null terminator
      JZ      .EP                                                       ; Jump to the end of the procedure if yes
      INT     0x10                                                      ; Print the character if not
      INC     BX                                                        ; Move to the next byte of the string
      JMP     .WriteString                                              ; Keep doing this until we find the null-terminator
    .EP:                                                                ; End of the procedure
      JMP     $                                                         ; Stay right here withoug returning to the calling procedure







; 
  ERRNoBootablePartition        DB      'No Bootable Partition', 0x00   ; Print when no bootable partition (!= 0x80) is found
  ERRDiskRead                   DB      'Disk Read Error', 0x00         ; Print when we couldn't read from the HDD due to an error
  ERRInvalidBootSector          DB      'Invalid Boot Sector', 0x00     ; Print when the boot signature 0xAA55 could not be found
; 
  TIMES   446 - ($-$$) DB 0x00                                          ; The bootstrap code should only be 446 bytes long
